#include <AfxWin.h>
#include <AfxExt.h>
#define INITGUID
#include <ddraw.h>
#include <d3d.h>
#include <d3drm.h>
#include <d3drmwin.h>
#include "SC98Win.h"
#include "resource.h"

#include <fstream>
#include <math.h>

#include "NetMgr.h"
#include "DemoMgr.h"

#include "ParamDlg.h"

#pragma comment (lib,"ddraw.lib")
#pragma comment (lib,"d3drm.lib")

const double X_MIN = -5000.00;
const double X_MAX = 5000.00;
const double Y_MIN = -4000.00;
const double Y_MAX = 4000.00;
const double Z_MIN = -5000.00;
const double Z_MAX = 5000.00;

/*Knox*/
const double x_min = 307086.5608; //53046.8048
const double x_max = 360133.3656;
const double y_min = 3865063.956; //59296.886
const double y_max = 3924360.842;
const double z_min = -5054622.029; //49398.151
const double z_max = -5005223.878;


/*Hunter
const double x_min = 308137.5889;
const double x_max = 360121.8057;
const double y_min = 3864970.872;
const double y_max = 3905121.246;
const double z_min = -5054477.452;
const double z_max = -5020496.771;
*/

const int numFaces = 10000;

static double DELTA = 10.0;


const D3DVALUE INITIAL_CAMERA_X = D3DVALUE(1692.0);
const D3DVALUE INITIAL_CAMERA_Y = D3DVALUE(-680.0);
const D3DVALUE INITIAL_CAMERA_Z = D3DVALUE(-692.0);

BEGIN_MESSAGE_MAP(SC98Win, RMWin)
	//{{AFX_MSG_MAP(SC98Win)
	ON_COMMAND(ID_RENDER_WIREFRAME, OnRenderWireframe)
	ON_COMMAND(ID_RENDER_FLAT, OnRenderFlat)
	ON_COMMAND(ID_RENDER_GOURAUD, OnRenderGouraud)
	ON_UPDATE_COMMAND_UI(ID_RENDER_FLAT, OnUpdateRenderFlat)
	ON_UPDATE_COMMAND_UI(ID_RENDER_GOURAUD, OnUpdateRenderGouraud)
	ON_UPDATE_COMMAND_UI(ID_RENDER_WIREFRAME, OnUpdateRenderWireframe)
	ON_WM_KEYDOWN()
	ON_COMMAND(ID_PARAMETERS, OnParameters)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

cVehUpdtMessageList* VehList;
Vehicle1* VehMessage;
cDetMessageList* DetList;
Detonation* DetMessage;


SC98Win::SC98Win()
{
	terrainMeshBuilder=0;
	tankMeshBuilder=0;

	int i=0;
	char **c;

	c = new char*[10];
	c[0] = "sc98.exe";

	ifstream params("params.dat");
	while(!params.eof())
	{
		i++;
		c[i] = new char[20];
		params.getline(c[i], 20, ' ');
	}

	VehList = new cVehUpdtMessageList;
	VehMessage = new Vehicle1;
	DetList = new cDetMessageList;
	DetMessage = new Detonation;

	gDemoMgr.Initialize(&i, c);
}


SC98Win::~SC98Win()
{
	terrainFrame->Release();
	terrainFrame=0;

	gDemoMgr.Shutdown();

	entity_frame_map::iterator i;
	for(i=efMap.begin(); i!=efMap.end(); i++)
	{
		(i->second)->Release();
		i->second=0;
	}
}


BOOL SC98Win::CreateScene()
{
	d3drm->CreateMeshBuilder(&terrainMeshBuilder);

	CreateTerrain(numFaces);

	terrainMeshBuilder->GenerateNormals();

	//--------- Vehicle Mesh ---------
	d3drm->CreateMeshBuilder(&tankMeshBuilder);
	tankMeshBuilder->Load("media\\tank.x", NULL, D3DRMLOAD_FROMFILE, NULL, NULL);
	tankMeshBuilder->SetQuality( D3DRMRENDER_FLAT);

	//--------- Plane Mesh ---------
	d3drm->CreateMeshBuilder(&planeMeshBuilder);
	planeMeshBuilder->Load("media\\f16.x", NULL, D3DRMLOAD_FROMFILE, NULL, NULL);
	planeMeshBuilder->SetQuality( D3DRMRENDER_FLAT);

	//------- Terrain frame --------
	d3drm->CreateFrame( scene, &terrainFrame );
	terrainFrame->AddVisual( terrainMeshBuilder );

	//-------- Animation ----------
	d3drm->CreateFrame( NULL, &blastFrame );
	d3drm->CreateAnimationSet(&blastAnim );
	blastAnim->Load("media\\blast.x", NULL, D3DRMLOAD_FROMFILE, NULL, NULL, blastFrame);

	//Delete this
/*				blastFrame->SetPosition(scene,
									D3DVALUE(INITIAL_CAMERA_X),
									D3DVALUE(INITIAL_CAMERA_Y),
									D3DVALUE(INITIAL_CAMERA_Z) );

			scene->AddChild(blastFrame);*/

	//-------- TEXTURE -------
	LPDIRECT3DRMTEXTURE texture;
	d3drm->LoadTexture( "media\\sky.bmp", &texture );
	scene->SetSceneBackgroundImage(texture);
	texture->Release();
	texture=0;

	// ---------AMBIENT LIGHT--------
	LPDIRECT3DRMLIGHT alight;
	d3drm->CreateLightRGB( D3DRMLIGHT_AMBIENT,
			D3DVALUE(0.6), D3DVALUE(0.6), D3DVALUE(0.7), 
			&alight);

	scene->AddLight( alight );
	alight->Release();
	alight=0;

	// ---------DIRECTIONAL LIGHT--------
	LPDIRECT3DRMLIGHT dlight;
	d3drm->CreateLightRGB( D3DRMLIGHT_DIRECTIONAL,
			D3DVALUE(1.00), D3DVALUE(1.00), D3DVALUE(1.00), 
			&dlight);

	LPDIRECT3DRMFRAME dlightframe;
	d3drm->CreateFrame( scene, &dlightframe );
	dlightframe->AddLight( dlight );
	dlightframe->SetOrientation( scene, 
			D3DVALUE(0), D3DVALUE(-1), D3DVALUE(10),
			D3DVALUE(0), D3DVALUE(1), D3DVALUE(0));
	dlight->Release();
	dlight=0;
	dlightframe->Release();
	dlightframe=0;

	//------ CAMERA-----------
	d3drm->CreateFrame( scene, &camera );
	camera->SetPosition( scene, INITIAL_CAMERA_X, INITIAL_CAMERA_Y, INITIAL_CAMERA_Z);
	d3drm->CreateViewport( device, camera, 0, 0, device->GetWidth(), device->GetHeight(), 
			&viewport);

	viewport->SetBack(D3DVALUE(50000));

	return TRUE;
}

void SC98Win::OnRenderWireframe() 
{
	if (terrainMeshBuilder)
		terrainMeshBuilder->SetQuality( D3DRMRENDER_WIREFRAME );
}

void SC98Win::OnRenderFlat() 
{
	if (terrainMeshBuilder)
		terrainMeshBuilder->SetQuality( D3DRMRENDER_FLAT);
}

void SC98Win::OnRenderGouraud() 
{
	if (terrainMeshBuilder)
		terrainMeshBuilder->SetQuality( D3DRMRENDER_GOURAUD);
}

void SC98Win::OnUpdateRenderWireframe(CCmdUI* pCmdUI) 
{
	if (terrainMeshBuilder)
	{
		D3DRMRENDERQUALITY meshquality = terrainMeshBuilder->GetQuality();
		pCmdUI->SetCheck( meshquality==D3DRMRENDER_WIREFRAME );
	}
}

void SC98Win::OnUpdateRenderFlat(CCmdUI* pCmdUI) 
{
	if (terrainMeshBuilder)
	{
		D3DRMRENDERQUALITY meshquality = terrainMeshBuilder->GetQuality();
		pCmdUI->SetCheck( meshquality==D3DRMRENDER_FLAT );
	}
}

void SC98Win::OnUpdateRenderGouraud(CCmdUI* pCmdUI) 
{
	if (terrainMeshBuilder)
	{
		D3DRMRENDERQUALITY meshquality = terrainMeshBuilder->GetQuality();
		pCmdUI->SetCheck( meshquality==D3DRMRENDER_GOURAUD );
	}
}

void SC98Win::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	//entity_frame_map::iterator i;
	static D3DVALUE cameraX = INITIAL_CAMERA_X;
	static D3DVALUE cameraY = INITIAL_CAMERA_Y;
	static D3DVALUE cameraZ = INITIAL_CAMERA_Z;

	static double temp=0;
	D3DVECTOR vec;

	switch ( nChar )
	{
	//Navigation - camera is moved with respect to it's last position
   	case VK_UP: //Move forward
		camera->SetPosition( camera, D3DVALUE(0), D3DVALUE(0), D3DVALUE(DELTA));
		camera->GetPosition(scene, &vec);
		cameraX = vec.x;
		cameraY = vec.y;
		cameraZ = vec.z;
		break;
	case VK_DOWN: //Move backward
		camera->SetPosition( camera, D3DVALUE(0), D3DVALUE(0), D3DVALUE(-DELTA));
		camera->GetPosition(scene, &vec);
		cameraX = vec.x;
		cameraY = vec.y;
		cameraZ = vec.z;
		break;
	case VK_HOME: //Move UP
		camera->SetPosition( camera, D3DVALUE(0), D3DVALUE(DELTA), D3DVALUE(0));
		camera->GetPosition(scene, &vec);
		cameraX = vec.x;
		cameraY = vec.y;
		cameraZ = vec.z;
		break;
	case VK_END: //Move DOWN
		camera->SetPosition( camera, D3DVALUE(0), D3DVALUE(-DELTA), D3DVALUE(0));
		camera->GetPosition(scene, &vec);
		cameraX = vec.x;
		cameraY = vec.y;
		cameraZ = vec.z;
		break;
	case VK_DELETE: //Move LEFT
		camera->SetPosition( camera, D3DVALUE(-DELTA), D3DVALUE(0), D3DVALUE(0));
		camera->GetPosition(scene, &vec);
		cameraX = vec.x;
		cameraY = vec.y;
		cameraZ = vec.z;
		break;
	case VK_NEXT: //Move RIGHT
		camera->SetPosition( camera, D3DVALUE(DELTA), D3DVALUE(0), D3DVALUE(0));
		camera->GetPosition(scene, &vec);
		cameraX = vec.x;
		cameraY = vec.y;
		cameraZ = vec.z;
		break;
	case VK_LEFT: //Turn left
		camera->AddRotation( D3DRMCOMBINE_BEFORE, D3DVALUE(0), D3DVALUE(1), D3DVALUE(0), D3DVALUE(-0.1));
		break;
	case VK_RIGHT: //Turn right
		camera->AddRotation( D3DRMCOMBINE_BEFORE, D3DVALUE(0), D3DVALUE(1), D3DVALUE(0), D3DVALUE(0.1));
		break;
	case VK_SPACE: //Turn right
		camera->AddRotation( D3DRMCOMBINE_BEFORE, D3DVALUE(1), D3DVALUE(0), D3DVALUE(0), D3DVALUE(-0.1));
		break;
	case VK_SHIFT: //Turn right
		camera->AddRotation( D3DRMCOMBINE_BEFORE, D3DVALUE(1), D3DVALUE(0), D3DVALUE(0), D3DVALUE(0.1));
		break;

	//To move camera along x,y,z axes of the scene
	case VK_NUMPAD8: //+z direction
		cameraZ+=D3DVALUE(DELTA);
		camera->SetPosition( scene, cameraX, cameraY, cameraZ );
		break;
	case VK_NUMPAD2://-z direction
		cameraZ-=D3DVALUE(DELTA);
		camera->SetPosition( scene, cameraX, cameraY, cameraZ);
		break;
	case VK_NUMPAD4: //-x direction
		cameraX-=D3DVALUE(DELTA);
		camera->SetPosition( scene, cameraX, cameraY, cameraZ );
		break;
	case VK_NUMPAD6://+x direction
		cameraX+=D3DVALUE(DELTA);
		camera->SetPosition( scene, cameraX, cameraY, cameraZ);
		break;
	case VK_NUMPAD9: //+y direction
		cameraY+=D3DVALUE(DELTA);
		camera->SetPosition( scene, cameraX, cameraY, cameraZ );
		break;
	case VK_NUMPAD3://-y direction
		cameraY-=D3DVALUE(DELTA);
		camera->SetPosition( scene, cameraX, cameraY, cameraZ);
		break;
	case VK_F1:
		blastAnim->SetTime(D3DVALUE(temp));
		temp++;
		break;

		//Remove these
	case VK_F2:
		terrainFrame->SetPosition(terrainFrame,D3DVALUE(DELTA),D3DVALUE(0.0),D3DVALUE(0.0));
		break;
	case VK_F3:
		terrainFrame->SetPosition(terrainFrame,D3DVALUE(-DELTA),D3DVALUE(0.0),D3DVALUE(0.0));
		break;
	case VK_F4:
		terrainFrame->SetPosition(terrainFrame,D3DVALUE(0.0),D3DVALUE(DELTA),D3DVALUE(0.0));
		break;
	case VK_F5:
		terrainFrame->SetPosition(terrainFrame,D3DVALUE(0.0),D3DVALUE(-DELTA),D3DVALUE(0.0));
		break;
	case VK_F6:
		terrainFrame->SetPosition(terrainFrame,D3DVALUE(0.0),D3DVALUE(0.0),D3DVALUE(DELTA));
		break;
	case VK_F7:
		terrainFrame->SetPosition(terrainFrame,D3DVALUE(0.0),D3DVALUE(0.0),D3DVALUE(-DELTA));
		break;
	case VK_F8:
		blastFrame->SetPosition(blastFrame,D3DVALUE(DELTA),D3DVALUE(0.0),D3DVALUE(0.0));
		break;
	case VK_F9:
		blastFrame->SetPosition(blastFrame,D3DVALUE(-DELTA),D3DVALUE(0.0),D3DVALUE(0.0));
		break;
	case VK_INSERT:
		blastFrame->SetPosition(blastFrame,D3DVALUE(0.0),D3DVALUE(DELTA),D3DVALUE(0.0));
		break;
	case VK_F11:
		blastFrame->SetPosition(blastFrame,D3DVALUE(0.0),D3DVALUE(-DELTA),D3DVALUE(0.0));
		break;
	case VK_F12:
		blastFrame->SetPosition(blastFrame,D3DVALUE(0.0),D3DVALUE(0.0),D3DVALUE(DELTA));
		break;
	case VK_ESCAPE:
		blastFrame->SetPosition(blastFrame,D3DVALUE(0.0),D3DVALUE(0.0),D3DVALUE(-DELTA));
		break;

	}
	
	RMWin::OnKeyDown(nChar, nRepCnt, nFlags);
}

BOOL SC98Win::CreateTerrain(int numFaces)
{
	LPDIRECT3DRMFACE lpFace;
	ifstream terrainData("terrain\\knox.dat");
	double x,y,z;
	int soilType=0;

	D3DCOLOR soilcolor[16] = {
					D3DRGB((181.00/255.00), (166.00/255.00), ( 66.00/255.00)),
					D3DRGB(( 84.00/255.00), ( 84.00/255.00), ( 84.00/255.00)),
					D3DRGB((202.00/255.00), (174.00/255.00), (162.00/255.00)),
					D3DRGB((159.00/255.00), (159.00/255.00), ( 95.00/255.00)),
					D3DRGB(( 35.00/255.00), (107.00/255.00), (190.00/255.00)),
					D3DRGB(( 50.00/255.00), (153.00/255.00), (204.00/255.00)),
					D3DRGB((209.00/255.00), (146.00/255.00), (117.00/255.00)),
					D3DRGB((209.00/255.00), (146.00/255.00), (117.00/255.00)),
					D3DRGB((217.00/255.00), (217.00/255.00), (243.00/255.00)),
					D3DRGB((192.00/255.00), (192.00/255.00), (192.00/255.00)),
					D3DRGB(( 35.00/255.00), (142.00/255.00), ( 35.00/255.00)),
					D3DRGB(( 47.00/255.00), ( 79.00/255.00), ( 79.00/255.00)),
					D3DRGB(( 47.00/255.00), ( 79.00/255.00), ( 79.00/255.00)),
					D3DRGB((140.00/255.00), (120.00/255.00), ( 56.00/255.00)),
					D3DRGB((234.00/255.00), (234.00/255.00), (174.00/255.00)),
					D3DRGB((216.00/255.00), (216.00/255.00), (191.00/255.00)),
					};

	/*D3DVALUE soilcolor[16][3]= { 
								{D3DVALUE(181.00/255.00), D3DVALUE(166.00/255.00), D3DVALUE( 66.00/255.00)},
								{D3DVALUE( 84.00/255.00), D3DVALUE( 84.00/255.00), D3DVALUE( 84.00/255.00)},
								{D3DVALUE( 92.00/255.00), D3DVALUE( 64.00/255.00), D3DVALUE( 52.00/255.00)},
								{D3DVALUE(159.00/255.00), D3DVALUE(159.00/255.00), D3DVALUE( 95.00/255.00)},
								{D3DVALUE( 35.00/255.00), D3DVALUE(107.00/255.00), D3DVALUE(190.00/255.00)},
								{D3DVALUE( 50.00/255.00), D3DVALUE(153.00/255.00), D3DVALUE(204.00/255.00)},
								{D3DVALUE(209.00/255.00), D3DVALUE(146.00/255.00), D3DVALUE(117.00/255.00)},
								{D3DVALUE(209.00/255.00), D3DVALUE(146.00/255.00), D3DVALUE(117.00/255.00)},
								{D3DVALUE(217.00/255.00), D3DVALUE(217.00/255.00), D3DVALUE(243.00/255.00)},
								{D3DVALUE(192.00/255.00), D3DVALUE(192.00/255.00), D3DVALUE(192.00/255.00)},
								{D3DVALUE( 35.00/255.00), D3DVALUE(142.00/255.00), D3DVALUE( 35.00/255.00)},
								{D3DVALUE( 47.00/255.00), D3DVALUE( 79.00/255.00), D3DVALUE( 79.00/255.00)},
								{D3DVALUE( 47.00/255.00), D3DVALUE( 79.00/255.00), D3DVALUE( 79.00/255.00)},
								{D3DVALUE(140.00/255.00), D3DVALUE(120.00/255.00), D3DVALUE( 56.00/255.00)},
								{D3DVALUE(234.00/255.00), D3DVALUE(234.00/255.00), D3DVALUE(174.00/255.00)},
								{D3DVALUE(216.00/255.00), D3DVALUE(216.00/255.00), D3DVALUE(191.00/255.00)},
							};*/

	for(int i=0; i<numFaces ; i++)
	{
		terrainMeshBuilder->CreateFace(&lpFace);

		for(int j=0; j<4; j++)
		{
			terrainData >> x;
			terrainData >> z;
			terrainData >> y;
			terrainData >> soilType;

			lpFace->AddVertex(D3DVALUE(ScaleX(x)), D3DVALUE(ScaleY(y)), D3DVALUE(ScaleZ(z)));
		}
		lpFace->SetColor(soilcolor[soilType]);
		//lpFace->SetColorRGB(soilcolor[soilType][0], soilcolor[soilType][1], soilcolor[soilType][2]);
	}

	return TRUE;
}


void SC98Win::OnIdle( long )
{
	static char buf[20];
	static float psi, theta, phi;
	static int count;
	static BOOL exploding = false;
	static float animTime = 0;
	static const float PI =  3.142857142857f;

	D3DVECTOR dirVec, upVec, tempVec;

	entity_frame_map::iterator i;

	int k;
	k = gDemoMgr.Run();

	switch(k)
	{
	case 1: // ESPdu received
		VehList = gDemoMgr.getVehList();
		VehMessage = VehList->getFirstMessage();

		i = efMap.find(VehMessage->mID.entityNum);

		if(i==efMap.end())
		{
			d3drm->CreateFrame( scene, &vehicleFrame );

			switch(VehMessage->mEtype.kind)
			{
				case 1:
				{
					switch(VehMessage->mEtype.domain)
					{
						case 1: //Land
						{
							switch(VehMessage->mEtype.category)
							{
								default:
									{
										vehicleFrame->AddVisual( tankMeshBuilder );
										break;
									}
							}
							break;
						}
						case 2: //Air
						{
							switch(VehMessage->mEtype.category)
							{
								default:
									{
										vehicleFrame->AddVisual( planeMeshBuilder );
										break;
									}
							}
							break;
						}
					}
					break;
				}
			}
	
			i = efMap.insert(efMap.begin(), entity_frame_map::value_type(VehMessage->mID.entityNum, vehicleFrame));
		}

		(i->second)->SetPosition(scene,
								 D3DVALUE(ScaleX(VehMessage->mLoc.x)),
								 D3DVALUE(ScaleY(VehMessage->mLoc.z)),
								 D3DVALUE(ScaleZ(VehMessage->mLoc.y)) );

		(i->second)->SetVelocity(scene,
								 D3DVALUE(VehMessage->mLinearVel.x/1000),
								 D3DVALUE(VehMessage->mLinearVel.z/1000),
								 D3DVALUE(VehMessage->mLinearVel.y/1000),
								 FALSE);


		psi = VehMessage->mOrientation.psi;
		theta = -VehMessage->mOrientation.theta;
		phi = VehMessage->mOrientation.phi;


		dirVec.x = D3DVALUE(cos(psi)*cos(theta));
		dirVec.y = D3DVALUE(sin(theta));
		dirVec.z = D3DVALUE(sin(psi)*cos(theta));

		tempVec.x = D3DVALUE(cos(psi)*cos(PI/2 + theta));
		tempVec.y = D3DVALUE(sin(PI/2 + theta));
		tempVec.z = D3DVALUE(sin(psi)*cos(PI/2 + theta));

		//upVec.x = 0;
		//upVec.y = 1;
		//upVec.z = -1;
		upVec = tempVec;

		D3DRMVectorRotate(&upVec, &tempVec, &dirVec, D3DVALUE(PI - VehMessage->mOrientation.phi));

		(i->second)->SetOrientation(scene, dirVec.x, dirVec.y, dirVec.z, upVec.x, upVec.y, upVec.z );
		//count++;
		break;

	case 2: // Detonation PDU received
		DetList = gDemoMgr.getDetList();
		DetMessage = DetList->getFirstMessage();

		i = efMap.find(DetMessage->mTargetEntity.entityNum);
		if(i!=efMap.end())
		{
			exploding = true;
			scene->DeleteChild(i->second);
			blastFrame->SetPosition(scene,
									D3DVALUE(ScaleX(DetMessage->mDetLocation.x)),
									D3DVALUE(ScaleY(DetMessage->mDetLocation.z)),
									D3DVALUE(ScaleZ(DetMessage->mDetLocation.y)) );

			scene->AddChild(blastFrame);
			animTime=0;
		}
		
		count++;
		_itoa(count, buf, 10);
		SetWindowText(buf);
		break;
	}

	if(exploding)
	{
		blastAnim->SetTime(D3DVALUE(animTime));
		animTime++;

		if(animTime==50)
		{
			scene->DeleteChild(blastFrame);
			exploding = false;
			animTime = 0;
		}
	}
}


double SC98Win::ScaleX(double x)
{
	return( ( ((x - x_min)/(x_max - x_min)) * (X_MAX - X_MIN) ) + X_MIN );
}

double SC98Win::ScaleY(double y)
{
	return( ( ((y - y_min)/(y_max - y_min)) * (Y_MAX - Y_MIN) ) + Y_MIN );
}

double SC98Win::ScaleZ(double z)
{
	return( ( ((z - z_min)/(z_max - z_min)) * (Z_MAX - Z_MIN) ) + Z_MIN );
}

void SC98Win::OnParameters() 
{
	// TODO: Add your command handler code here
	ParamDlg pd;
	pd.m_delta = DELTA;
	pd.DoModal();
	DELTA = pd.m_delta;
}
